#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for artella
"""

from __future__ import print_function, division, absolute_import

from pkgutil import extend_path

__path__ = extend_path(__path__, __name__)
